#____________________________________________________________________________#
#|                                                                          |#
#| This file contains all procedures linked to the attributes of canvas-    |#
#| objects.                                                                 |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#
# Sets a list to hold values containing information about the attributes of
# an object.
# 
# args    : 1. Reference to the object.
# returns : A list of attributes.

proc Attribute_Tags {itemIndex} {

# Allow access to globals controlling attributes
    global _Colour _Line_Width _Style _Absolute_URL

# Identify object type
    set type [.can.c1 type $itemIndex]

# Get a list of the tags for the object
    set tagList [.can.c1 gettags $itemIndex]
    set oldAttribs [list ""]

# Get a list of the old Attribute-TagList
    set oldAttribs [Find_Attributes $tagList]

# If the object has been annotated already, index these.
    set comments [lindex $oldAttribs 3]
    set url [lindex $oldAttribs 4]

# If the current url value is empty - in the case of copies - then set it
# to the "_Absolute_URL" path.
    if {$url == ""} {
        set url $_Absolute_URL
    }

    set itemAttributes [list $type $_Colour $_Line_Width $comments $url]

    return $itemAttributes
}

#============================================================================#
# Identifies the tag containing a list of the attributes of an object from
# its taglist.
#
# args    : 1. List of tags for selected object.
# returns : A list containing the attributes.

proc Find_Attributes {allTags} {

    set attribs ""

# Loop through all tags and find the one holding attribute-values.
# This will be the list element which contains more than 1 item.
    foreach tag $allTags {
        if {[llength $tag] > 1} {
            set attribs $tag
        }
    }

# Return the attribute list
    return $attribs
}

#============================================================================#
# Deselects any currently selected object and selects a new object.
# args   : 1. Reference to object to be selected.

proc Auto_Select {thisItem} {

    global _Auto_Raise

# If "selected" is parsed into this procedure, it is so the selected object can
# be displayed to show this after a function has been carried out.
# In this case the tag "selected" should not be removed from the list of tags.
    if {$thisItem != "selected"} {

# If not the selected object, then deselect the currently selected, and select
# the object specified by "thisItem".
        Deselect_Selected
        .can.c1 addtag selected withtag $thisItem
        .can.c1 dtag $thisItem
        .can.c1 dtag selected item
        if {$_Auto_Raise} {
            .can.c1 raise selected all
        }
    } else {

# If the object is the currently selected, remove any visual-aids from the
# canvas, before creating the one to show the selected object.
        Remove_VisualAids
    }

# Identify the coordinates of an approximate bounding-box for the object.
    set bboxCoords [.can.c1 bbox selected]

# Draw the bounding-box for the object on the canvas.
    eval ".can.c1 create rectangle $bboxCoords -outline purple \
        -tags visualaid"
# Call procedure to create a series of points around the edge of
# the bounding-box.
    Create_Bbox_Points $bboxCoords

    Configure_Test_URLbutton

}

#============================================================================#
# Create a series of points around the bounding-box.  These can be used to
# carry out certain edit-functions.
# args   : 1. A list of the coordinates of the bounding-box.

proc Create_Bbox_Points {rectCoords} {

    set x1 [lindex $rectCoords 0]
    set y1 [lindex $rectCoords 1]
    set x2 [lindex $rectCoords 2]
    set y2 [lindex $rectCoords 3]

# Calculate mid-points on the bounding-box.
    catch {set mx [expr ($x1+($x2-$x1)/2)]}
    catch {set my [expr ($y1+($y2-$y1)/2)]}

# Draw the points on the canvas.
    eval ".can.c1 create rectangle $x1 $y1 $x1 $y1 -width 5 \
        -outline green -tags selpt1"
    .can.c1 addtag visualaid withtag selpt1

    eval ".can.c1 create rectangle $mx $y1 $mx $y1 -width 5 \
        -outline green -tags selpt2"
    .can.c1 addtag visualaid withtag selpt2

    eval ".can.c1 create rectangle $x2 $y1 $x2 $y1 -width 5 \
        -outline green -tags selpt3"
    .can.c1 addtag visualaid withtag selpt3

    eval ".can.c1 create rectangle $x1 $my $x1 $my -width 5 \
        -outline green -tags selpt4"
    .can.c1 addtag visualaid withtag selpt4

    eval ".can.c1 create rectangle $x2 $my $x2 $my -width 5 \
        -outline green -tags selpt5"
    .can.c1 addtag visualaid withtag selpt5

    eval ".can.c1 create rectangle $x1 $y2 $x1 $y2 -width 5 \
        -outline green -tags selpt6"
    .can.c1 addtag visualaid withtag selpt6

    eval ".can.c1 create rectangle $mx $y2 $mx $y2 -width 5 \
        -outline green -tags selpt7"
    .can.c1 addtag visualaid withtag selpt7

    eval ".can.c1 create rectangle $x2 $y2 $x2 $y2 -width 5 \
        -outline green -tags selpt8"
    .can.c1 addtag visualaid withtag selpt8

}
   
#============================================================================#
# Changes the appearance of the selected canvas object.
# Invoked by buttons under the "Attributes" menu.

proc Change_Selected_Appearance {} {

    global _Style

# Get a list of the Attributes for the selected object.
    set newAttribs [Attribute_Tags selected]

# If there is no currently selected object, it will hold a null-list.
# Carry out the procedure only if there is no null-list
    if {[lindex $newAttribs 0] != ""} {
        set type [lindex $newAttribs 0]
        set coords [.can.c1 coords selected]
        set colour [lindex $newAttribs 1]
        set lineWidth [lindex $newAttribs 2]

# If the change in appearance is its draw-style, set the line-width accordingly.
        if {$_Style == "fill" && $lineWidth != 0} {
            uplevel #0 "set _Line_Width 0"
            set lineWidth 0
        }

        if {$lineWidth == 0 && $_Style == "outline"} {
            uplevel #0 "set _Line_Width 4"
            set lineWidth 4
        }

# Delete the selected object
        .can.c1 delete selected

# Draw a copy of it with new appearance
        Create_Canvas_Object $type $coords $_Style $colour $lineWidth

        for {set a [expr [llength [.can.c1 gettags selected]]-1]} \
            {$a>=1} {incr a -1} {

            set tagToDelete [lindex [.can.c1 gettags selected] $a]
            .can.c1 dtag selected $tagToDelete
        }

        .can.c1 addtag $newAttribs withtag selected
    }
}

#============================================================================#
# Sets the radiobuttons under the Attributes-menu to the values of the 
# currently selected object.

proc Set_Attrib_Radiobuttons {} {

    global _Colour _Style _Line_Width

    set selectedAttribs [Find_Attributes [.can.c1 gettags selected]]

    set _Colour [lindex $selectedAttribs 1]
    set _Line_Width [lindex $selectedAttribs 2]

    if {$_Line_Width == 0} {
        set _Style fill
    } else {
        set _Style outline
    }
}

