#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to set bindings for all the "Create"       |#
#| object functions.  These include creating Rectangles, Circles and        |#
#| Polygon objects.                                                         |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#
# Invoked from "proc Set_Correct_Binds".
# Calls relevant procedures for the "Create" options with a function 
# identifying string.
# args :
#       1. String containing contents of global variable linked to Create/Edit
#          options.
 
proc Create_Objects {selectedFunction} {

# Configure the "Test_URL" button to state disabled.
    .can.menu.test configure -state disabled

# Depending which radiobutton is currently selected, call procedures to set
# appropriate canvas bindings.
    switch $selectedFunction {
        RECTANGLE {
            Create_Object_BP1 rectangle
            Drag_Motion rectangle
            Create_Object_BR1 rectangle
        }
        CIRCLE_RAD {
            Create_Object_BP1 radius
            Drag_Motion radius
            Create_Object_BR1 radius
        }
        CIRCLE_BB {
            Create_Object_BP1 bbox
            Drag_Motion bbox
            Create_Object_BR1 bbox
        }
        POLYGON {
            Create_Object_BP1 polygon
            Create_Object_BP2 
        }
    }

# Sets an appropriate guidance message to be displayed in the message widget
# under the canvas.
    switch $selectedFunction {
        CIRCLE_RAD {
            set msg " <ButtonPress-1>    -  Identifies centre-point \n\
                <B1-Motion>           -  To outer edge \n <ButtonRelease-1> - \
                Creates the object"
            Configure_Message_Widget $msg
        }
        POLYGON {
            set msg " <ButtonPress-1> - Identifies each point \n\
               <ButtonPress-2> - Closes and draws object \n"
            Configure_Message_Widget $msg
        }
        default {
            Set_Message 2
        }
    }
}

#============================================================================#

# Invoked from "proc Create_Objects"
# Sets the binding for <ButtonPress-1> in the canvas on any one of the "Create"
# object functions.
# args :
#       1. String containing identifier to which operation to carry out.

proc Create_Object_BP1 {type} {

# Set var to hold coordinates of the object being created.
    uplevel #0 {set coords ""}

# Depending which operation is selected set anchor for the visual-aid
    switch $type {
        rectangle
            -
        bbox {
            bind .can.c1 <ButtonPress-1> {
                set creating 0
                set coords [list [.can.c1 canvasx %x] [.can.c1 canvasy %y]]
            }
        }
        radius {
            bind .can.c1 <ButtonPress-1> {
                set creating 0
                set centre_x [.can.c1 canvasx %x]
	        set centre_y [.can.c1 canvasy %y]
            }
        }
        polygon {
            bind .can.c1 <ButtonPress-1> {
# Remove any previuosly drawn visual-aids from the canvas
                Remove_VisualAids
# Append the new coordinates onto the list of coordinates.
                set coords [concat $coords \
                    [.can.c1 canvasx %x] [.can.c1 canvasy %y]]
# Create a visual-aid for the user, connecting points together.
                if {[llength $coords] >= 3} {
                    eval ".can.c1 create line $coords -fill $_Colour \
                        -tags visualaid"
                }
            }
        }
    }
}

#============================================================================#

# Invoked from "proc Create_Objects"
# This is only called if the object being created is not a polygon.
# It creates a visual-aid on the canvas during the creation operation.
# args :
#       1. String containing identifier to which operation to carry out.

proc Drag_Motion {type} {

# For all operations the previously drawn visual-aid must be removed
    bind .can.c1 <B1-Motion> {
        if {[info exists creating]} {
            if {$creating != 1} {set creating 1}
        }
        Remove_VisualAids
    }

# The rest of the binding is dependant on which operation is selected
    switch $type {
        rectangle {
            bind .can.c1 <B1-Motion> {+
                eval ".can.c1 create rectangle $coords [.can.c1 canvasx %x] \
                    [.can.c1 canvasy %y] -outline $_Colour -width 1 \
                    -tags visualaid"
            }
        }
        bbox {
            bind .can.c1 <B1-Motion> {+
                set dx [expr ([.can.c1 canvasx %x]-[lindex $coords 0])]

                set diffx [expr [lindex $coords 0]+$dx]
                set diffy [expr [lindex $coords 1]+$dx]

                eval ".can.c1 create oval $coords $diffx $diffy \
                    -outline $_Colour -width 1 -tags visualaid"
            }
        }
        radius {
            bind .can.c1 <B1-Motion> {+
                set dx [expr ([.can.c1 canvasx %x]-$centre_x)]

# Set a temporary variable to hold the return value of the procedure which
# calculates the coordinates of the visual-aid for Radius-defined circles
                set edges [Calculate_Radius $dx $centre_x $centre_y]

                eval ".can.c1 create oval $edges -outline $_Colour \
                    -width 1 -tags visualaid"
            }
        }
    }
}

#============================================================================#

# Invoked from "proc Create_Objects"
# This is only called if the object being created is not a polygon.
# It creates the binding for <ButtonRelease-1> in the canvas which sets 
# the coordinate-list of the object being created, and calls the procedure
# to create the object.
# args :
#       1. String containing identifier to which operation to carry out.

proc Create_Object_BR1 {type} {

# Depending which option has been selected, set appropriate binding
    switch $type {
        rectangle {
            bind .can.c1 <ButtonRelease-1> {
                if {[info exists creating] && $creating == 1} {
                    set coords [concat $coords [.can.c1 canvasx %x] \
                        [.can.c1 canvasy %y]]

# Call procedure to draw the object on the canvas
                    Create_Canvas_Object rectangle $coords $_Style $_Colour \
                        $_Line_Width
                }
            }
        }
        bbox {
            bind .can.c1 <ButtonRelease-1> {
                if {[info exists creating] && $creating == 1} {
                    set coords [concat $coords $diffx $diffy]

# Call procedure to draw the object on the canvas
                    Create_Canvas_Object oval $coords $_Style $_Colour \
                        $_Line_Width
                }
            }
        }
        radius {
            bind .can.c1 <ButtonRelease-1> {
                if {[info exists creating] && $creating == 1} {
                    set coords $edges

# Call procedure to draw the object on the canvas
                    Create_Canvas_Object oval $coords $_Style $_Colour \
                        $_Line_Width
                }
            }
        }
    }


    bind .can.c1 <ButtonRelease-1> {+
        set attributeList [Attribute_Tags selected]
        .can.c1 addtag $attributeList withtag selected
    }

# The following is carried out on completion of the "Create" operation, so 
# add this on to the end of the ButtonRelease-1 binding no-matter which 
# option has been selected.  It  resets to the default canvas bindings.
    bind .can.c1 <ButtonRelease-1> {+
        if {![info exists creating] || $creating == 0} {
        } else {
# Remove all canvas bindings.  This leaves the canvas in a fixed state.
            Nullify_All_Bindings
# Force annotation of new object if option is selected.
            if {$_Force_Annotation} {
                Annotate_Selected
                Configure_Test_URLbutton
            }
# Reset to the default canvas bindings after one cycle, if Single_Edit_Mode
# checkbutton is on.
            if {$_Single_Edit_Mode} {
                Reset_To_Default_Binds
            } else {
                Reset_To_Function_Binds
            }
            unset creating
        }
    }
}

#============================================================================#

# Invoked from "proc Create_Objects".
# This procedure is only called when the "Create Polygon" option has been
# selected.
# Creates the binding used to close the polygon.

proc Create_Object_BP2 {} {

    bind .can.c1 <Button-2> {

# Only if the polygon has 3 or more points close it and draw.
        if {[llength $coords] >= 4} {

# Append the end coordinates onto Coordinate-List.
            set coords [concat $coords \
                [.can.c1 canvasx %x] [.can.c1 canvasy %y]]

# Call procedure to draw the object on the canvas.
            Create_Canvas_Object polygon $coords $_Style $_Colour $_Line_Width

# Set the attribute list for the new object and add it to the list of tags.
            set attributeList [Attribute_Tags selected]
            .can.c1 addtag $attributeList withtag selected

# Reset to the default bindings if checkbutton is off (_Single_Edit_Mode)
            Nullify_All_Bindings
            if {$_Force_Annotation} {
                Annotate_Selected
                Configure_Test_URLbutton
            }
            if {$_Single_Edit_Mode} {
                Reset_To_Default_Binds
            } else {
                Reset_To_Function_Binds
            }
        }
    }
}

