#____________________________________________________________________________#
#|                                                                          |#
#| This file contains all procedures to set the default-canvas-bindings.    |#
#| These include : Move Object, Scale Object, Select New Object and         |#
#| Deselect Currently Selected.                                             |#
#| Procedures :                                                             |#
#| 1. proc Which_Default    - calls relevant procedures to create the       |#
#|                            create the appropriate canvas bindings.       |# 
#| 2. proc Default_BP1      - creates bindings for all <ButtonPress-1>      |#
#|                            sequences in the canvas.                      |#
#| 3. proc Default_B1Motion - creates bindings for all <B1-Motion>          |#
#|                            sequences in the canvas.                      |#
#| 4. proc Default_BR1      - creates bindings for all <ButtonRelease-1>    |#
#|                            sequences in the canvas.                      |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#
# This procedure is invoked every time "proc Set_Correct_Binds" is called
# with an empty string.
# It identifies which of the default-bindings the user wants to execute 
# - from the users actions - and calls procedures to set up correct bindings.

proc Which_Default {} {

# Binding for the selection of the MoveSelectedObject option.
    .can.c1 bind selected <ButtonPress-1> {

# Call procedure to set relevant bindings for <ButtonPress-1>, <B1-Motion>
# and <ButtonRelease-1> 
        Default_BP1 moveSelected
        Default_B1Motion moveSelected
        Default_BR1 moveSelected
    }

# Binding for the selection of the ScaleSelectedObject binding.
    .can.c1 bind visualaid <ButtonPress-1> {

        Default_BP1 scaleSelected
        Default_B1Motion scaleSelected
        Default_BR1 scaleSelected
    }


# Binding for the selection  or deselection of objects on the canvas.
    bind .can.c1 <ButtonPress-1> {
        Default_BP1 selectFunctions
    }
}

#============================================================================#
# Sets up the binding for <ButtonPress-1> in the canvas.  There are four
# different default-actions (Move Selected, Scale Selected, Select New and
# Deselect Selected).
# args :
#       1. A string identifying which operation has been selected, from the
#          users action.  Used to set up the correct binding.

proc Default_BP1 {operation} {

# The following binding sets some values from the canvas.  These are required
# in the "Move Selected" and "Scale Selected" bindings only in "Create".
# This is also required in some of the "Edit" options.  If so, this procedure
# is called with "operation" containing the string 'fromEditting'
    switch $operation {
        moveSelected
            -
        scaleSelected
            -
        fromEditting {
            bind .can.c1 <ButtonPress-1> {
                set tagList [.can.c1 gettags selected]

                set type [.can.c1 type selected]
                set current_x [.can.c1 canvasx %x]
                set current_y [.can.c1 canvasy %y]
                set start_x $current_x
                set start_y $current_y
            }
        }
    }

# Switch statement sets up the correct binding depending on the value of the 
# string contained in "operation".
    switch $operation {

        moveSelected {
# Removes the bounding-box and its points from the canvas and creates an 
# outline of the selected object that can be dragged around with <B1-Motion>
# The "+" at the beginning of the command appends this command on the end
# of any command already existing for the sequence specified.
            bind .can.c1 <ButtonPress-1> {+
                Remove_VisualAids
                set coordinates [.can.c1 coords selected]

                switch $type {
                    oval
                        -
                    rectangle {
                        eval ".can.c1 create $type $coordinates \
                            -outline purple -tags visualaid"
                    }
                    polygon
                        -
                    line {
                        eval ".can.c1 create line $coordinates \
                            -fill purple -tags visualaid"
                    }
                }
            }
        }

        scaleSelected {
# Identifies which point has been selected, and sets up the anchor point
# that scaling will be executed around.
            bind .can.c1 <ButtonPress-1> {+
                .can.c1 addtag scalept closest \
                    [.can.c1 canvasx %x] [.can.c1 canvasy %y] 1 \
                    "withtag visualaid"

                set thisPoint [lindex [.can.c1 gettags scalept] 0]

# Sets variable to the return value of procedure to calculate anchor-point
                set anchorCoords [Set_Scaling_Anchor $thisPoint]
                set x0 [lindex $anchorCoords 0]
                set y0 [lindex $anchorCoords 1]
                set x1 [lindex $anchorCoords 2]
                set y1 [lindex $anchorCoords 3]

# x1 and y1 sometimes hold null values (depending which point has been
# selected by user.  If this is the case, set them to default values.
                if {$x1 == ""} {
                    set x1 [.can.c1 canvasx %x]
                }
                if {$y1 == ""} {
                    set y1 [.can.c1 canvasy %y]
                }
                set anchorCoords [list $x0 $y0 $x1 $y1]
            }

        }

        selectFunctions {
# The sequence-of-events for the selection and deselection of objects is 
# very similar.  To distinguish between the two, get the tags of the object
# that the cursor is in.  From this identify if the object has an attribute-
# list.  If there is no object (or the object is the "background photo-widget),
# this will be an empty string, so deselect the currently selected object.
# If this is not an empty string, the user wishes to select a new object, so
# identify the object and call procedure to select it.
            set tagList [.can.c1 gettags current]

            if {[Find_Attributes $tagList] == ""} {
                Deselect_Selected
                .can.menu.test configure -state disabled
            } else {
                .can.c1 addtag toSelect withtag current
                Auto_Select toSelect
                Set_Attrib_Radiobuttons

                Configure_Test_URLbutton

            }

            Reset_To_Default_Binds
        }
    }
}

#============================================================================#
# Sets up the bindings for <B1-Motion> in the canvas.  There are a possible
# two different commands.
# args :
#       1. A string identifying which operation has been selected, from the
#          users action.  Used to set up the correct binding.

proc Default_B1Motion {operation} {

    switch $operation {
# Calculates difference in position since last move(?) and moves the outline
# by this amount.
        moveSelected {
            bind .can.c1 <B1-Motion> {
                set dx [expr ([.can.c1 canvasx %x]-$current_x)]
                set dy [expr ([.can.c1 canvasy %y]-$current_y)]
                set current_x [.can.c1 canvasx %x]
                set current_y [.can.c1 canvasy %y]
                eval ".can.c1 move visualaid $dx $dy"
            }
        }
        scaleSelected {
# Removes any previously drawn visual-aids (if any) and creates a new one, in 
# the form of a box, anchored at the position specified from which point was
# selected, to the current canvas-coordinates of the cursor.  The rescaled
# object will fit into this box.
# NB - Circle (oval) type objects cannot be scaled by selecting any of the 
#      corner points.  This is done in order that the aspect-ratio will be
#      preserved.
            bind .can.c1 <B1-Motion> {
                Remove_VisualAids

                if {$thisPoint=="selpt2" || $thisPoint=="selpt7"} {
                    if {$type=="oval"} {
                        set tx [expr ($x0+([.can.c1 canvasy %y]-$y0))]
                        eval ".can.c1 create rectangle $x0 $y0 $tx \
                            [.can.c1 canvasy %y] -outline $_Colour -width 1 \
                                -tags visualaid"
                    } else {
                        eval ".can.c1 create rectangle $x0 $y0 $x1 \
                            [.can.c1 canvasy %y] -outline $_Colour -width 1 \
                            -tags visualaid"
                    }
                } else {
                    if {$thisPoint=="selpt4" || $thisPoint=="selpt5"} {
                        if {$type=="oval"} {
                            set ty [expr ($y0+([.can.c1 canvasx %x]-$x0))]
                            eval ".can.c1 create rectangle $x0 $y0 \
                                [.can.c1 canvasx %x] $ty -outline $_Colour \
                                -width 1 -tags visualaid"
                        } else {
                            eval ".can.c1 create rectangle $x0 $y0 \
                                [.can.c1 canvasx %x] $y1 -outline $_Colour \
                                -width 1 -tags visualaid"
                        }
                    } else {
                        if {$type=="oval"} {} else {
                            eval ".can.c1 create rectangle $x0 $y0 \
                                [.can.c1 canvasx %x] [.can.c1 canvasy %y] \
                                -outline $_Colour -width 1 -tags visualaid"
                        }
                    }
                }
            }
        }
    }
}


#============================================================================#
# Sets up the bindings for <ButtonRelease-1> in the canvas.
# There are a possible two different commands.
# args :
#       1. A string identifying which operation has been selected, from the
#          users action.  Used to set up the correct binding.

proc Default_BR1 {operation} {

    switch $operation {
# Sets the end coordinates of the movement, and calls procedure to calculate
# the difference between start and end.
# The selected object is then moved the calculated amount, and reselected to
# show the bounding-box and points that surround the selected object.
        moveSelected {
            bind .can.c1 <ButtonRelease-1> {
                set end_x [.can.c1 canvasx %x]
                set end_y [.can.c1 canvasy %y]

                set dx [Calculate_Difference $start_x $end_x]
                set dy [Calculate_Difference $start_y $end_y]

                .can.c1 move selected $dx $dy
                Auto_Select selected

                Reset_To_Default_Binds
            }
        }
# Removes the box used as a visual-aid, and calls procedure to calculate the 
# scale-factors to use in the x and y planes.
# These values are amended, and the scaling carried out on the selected object.
# The selected object is then reselected and the default bindings reset.
        scaleSelected {
            bind .can.c1 <ButtonRelease-1> {
                Remove_VisualAids

                set xscaleFactor [Calculate_Scale_Factor x $anchorCoords \
                    [.can.c1 canvasx %x]]
                set yscaleFactor [Calculate_Scale_Factor y $anchorCoords \
                    [.can.c1 canvasy %y]]

                .can.c1 dtag scalept

                switch $thisPoint {
                    selpt2
                        -
                    selpt7 {
                        if {$type == "oval"} {
                            set xscaleFactor $yscaleFactor
                        } else {
                            set xscaleFactor 1.0
                        }
                    }
                    selpt4
                        -
                    selpt5 {
                        if {$type == "oval"} {
                            set yscaleFactor $xscaleFactor
                        } else {
                            set yscaleFactor 1.0
                        }
                    }
                    selpt1
                        -
                    selpt3
                        -
                    selpt6
                        -
                    selpt8 {
                        if {$type == "oval"} {
                            set xscaleFactor 1.0
                            set yscaleFactor 1.0
                        }
                    }
                }

                eval ".can.c1 scale selected $x0 $y0 \
                    $xscaleFactor $yscaleFactor"
                Auto_Select selected

                Reset_To_Default_Binds
            }
        }

    }
}


