#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to carry out all calculations, etc. in     |#
#| all procedures to do with the default-canvas bindings.                   |#
#| Called when relevant.                                                    |#
#| Procedures :                                                             |#
#| 1. proc Calculate_Difference   - Calculates difference between a start-  |#
#|                                  value and an end-value.                 |#
#| 2. proc Set_Scaling_Anchor     - Calculates an anchor-point for the      |#
#|                                  scale-function.                         |#
#| 3. proc Calculate_Scale_Factor - Calculates a scale-factor for the       |#
#|                                  scale-function.                         |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#
# Calculates difference between a start-value and an end-value.
# args :
#       1. start-value
#       2. end-value
# returns : The difference

proc Calculate_Difference {coord1 coord2} {

    set coordDiff [expr ($coord2-$coord1)]

    return $coordDiff
}

#============================================================================#
# Calculates an anchor-point for the scale-function.
# args :
#       1. Which point on the bounding-box has been selected to scale with.
# returns : A list of coordinates used to set appropriate anchor.
proc Set_Scaling_Anchor {pointID} {

    set coords [.can.c1 bbox selected]
    set x0 ""
    set y0 ""
    set x1 ""
    set y1 ""

    switch $pointID {
        selpt1 {
            set x0 [lindex $coords 2]
            set y0 [lindex $coords 3]
        }
        selpt2 {
            set x0 [lindex $coords 2]
            set y0 [lindex $coords 3]
            set x1 [lindex $coords 0]
        }
        selpt3 {
            set x0 [lindex $coords 0]
            set y0 [lindex $coords 3]
        }
        selpt4 {
            set x0 [lindex $coords 2]
            set y0 [lindex $coords 3]
            set y1 [lindex $coords 1]
        }
        selpt5 {
            set x0 [lindex $coords 0]
            set y0 [lindex $coords 1]
            set y1 [lindex $coords 3]
        }
        selpt6 {
            set x0 [lindex $coords 2]
            set y0 [lindex $coords 1]
        }
        selpt7 {
            set x0 [lindex $coords 0]
            set y0 [lindex $coords 1]
            set x1 [lindex $coords 2]
        }
        selpt8 {
            set x0 [lindex $coords 0]
            set y0 [lindex $coords 1]
        }
    }

    set anchorPoints [list $x0 $y0 $x1 $y1]
    return $anchorPoints
}

#============================================================================#
# Calculates a scale-factor for the scale-function.
# args :
#       1. Which plane the scaling is being calculated for.
#       2. A list of coordinates used for the anchor.  This identifies the 
#          original coordinates of the selected object (prior to scale).
#       3. The coordinate on the canvas where <ButtonRelease-1> occured.
#          (The position to scale to).
# returns : value to scale by.

    #/-------------------------------------------------------------------\#
    #|###################################################################|#
    #|# Tcl does not support floating-point numbers, so by taking a     #|#
    #|# percentage value as a string and splitting to effectively give  #|#
    #|# a scaling factor to two decimal places, this can be overcome.   #|#
    #|###################################################################|#
    #\-------------------------------------------------------------------/#

proc Calculate_Scale_Factor {scalePlane anchorPoints canCoord} {

# Get a percentage value of the scaling-factor (NB// *100), in the
# relevant plane.
    if {$scalePlane == "x"} {
        set x0 [lindex $anchorPoints 0]
        set x1 [lindex $anchorPoints 2]

        set percentage [expr ([expr ($canCoord-$x0)*100]/[expr ($x1-$x0)])]
    } else {
        set y0 [lindex $anchorPoints 1]
        set y1 [lindex $anchorPoints 3]

        set percentage [expr ([expr ($canCoord-$y0)*100]/[expr ($y1-$y0)])]
    }

# Determine the length of the string containing this value.
    set sfLength [expr ([string length $percentage]-1)]

# Set variable to hold the last two elements of the string.
# This is the decimal part of the scaling factor.
    set sfDecimal [string range $percentage [expr ($sfLength-1)] $sfLength]

# If the string is more than 2 elements long, set a variable to hold all
# previous characters in the correct order.  This is the integer part of the
# scaling-factor.
    if {[expr ($sfLength-1)] != 0} {
        set sfInt [string range $percentage 0 [expr ($sfLength-2)]]

# If the first character of the integer part is a "-", set the string to "1.0"
        set sfInt1 [string range $sfInt 0 0]
            if {$sfInt1 == "-"} {
               set sfInt 1
               set sfDecimal 0
            }
    } else {
        set sfInt 0
    }

    set sfString "$sfInt.$sfDecimal"
    return $sfString
}

