#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to set bindings for all the "Create"       |#
#| object functions.  These include creating Rectangles and Circle type     |#
#| objects.                                                                 |#
#|                                                                          |#
#| Procedures :                                                             |#
#| 1. proc Edit_Objects    - calls relevant procedures to create the        |#
#|                           create the appropriate canvas bindings.        |#
#| 2. proc Edit_Object_BP1 - creates bindings for all <ButtonPress-1>       |#
#|                           sequences in the canvas.                       |#
#| 3. proc Edit_Object_BR1 - creates bindings for all <ButtonRelease-1>     |#
#|                           sequences in the canvas.                       |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#
# Invoked from "proc Set_Correct_Binds".
# args :
#       1. String containing contents of global variable linked to Create/Edit
#          options.

proc Edit_Objects {selectedFunction} {

    switch $selectedFunction {
        COPY_SELECTED {
# The bindings for <ButtonPress-1> and <B1-Motion> are identical to those for
# the default-binding for moving the selected object, so invoke these.
            Default_BP1 moveSelected
            Default_B1Motion moveSelected
            Edit_Object_BR1 copySelected
        }
        FLIP_ABOUT-X
            -
        FLIP_ABOUT-Y {
            Edit_Object_BP1 $selectedFunction
            Edit_Object_BP1 bothFlips
            Edit_Object_BR1 bothFlips
        }
        MOVE_POINT 
            -
        ADD_POINT
            -
        DELETE_POINT {
            set check [Check_Polygon]
            if {$check} {
                Create_Coordinate_Points

                switch $selectedFunction {
                    MOVE_POINT {
                        Edit_Polygon_BP1 $selectedFunction
                        Edit_Polygon_B1M 
                        Edit_Polygon_BR1 $selectedFunction
                    }
                    ADD_POINT {
                        Edit_Polygon_BP1 $selectedFunction
                        Edit_Polygon_B1M 
                        Edit_Polygon_BR1 $selectedFunction
                    }
                    DELETE_POINT {
                        Edit_Polygon_BP1 $selectedFunction
                        Edit_Polygon_BR1 $selectedFunction
                    }
                }
            } else {
                Reset_To_Default_Binds
            }
        }
    }
}

#============================================================================#

# Invoked from "proc Edit_Objects"
# Sets the binding for <ButtonPress-1> in the canvas on any one of the
# "Create" object functions.
# args :
#       1. String containing identifier to which operation to carry out.

proc Edit_Object_BP1 {operation} {

# Depending which operation is selected, set appropriate bindings.
    switch $operation {
        FLIP_ABOUT-X {
            .can.c1 bind visualaid <ButtonPress-1> {
                set coords [.can.c1 coords selected]
                set coordLength [llength $coords]

                set mirror_x [.can.c1 canvasx %x]
                for {set a 0} {$a<$coordLength} {incr a 2} {
                    set new_x [expr ($mirror_x+($mirror_x-[lindex $coords $a]))]
                    set coords [lreplace $coords $a $a $new_x]
	        }
            }
        }
        FLIP_ABOUT-Y {
            .can.c1 bind visualaid <ButtonPress-1> {
                set coords [.can.c1 coords selected]
                set coordLength [llength $coords]

                set mirror_y [.can.c1 canvasy %y]
                for {set a 1} {$a<$coordLength} {incr a 2} {
                    set new_y [expr ($mirror_y+($mirror_y-[lindex $coords $a]))]
                    set coords [lreplace $coords $a $a $new_y]
                }
            }
        }
        bothFlips {
            .can.c1 bind visualaid <ButtonPress-1> {+
                set type [.can.c1 type selected]

                set selectedAttribs [Find_Attributes [.can.c1 gettags selected]]

                set colour [lindex $selectedAttribs 1]
                set linewidth [lindex $selectedAttribs 2]
                if {$linewidth == 0} {
                    set style fill
                } else {
                    set style outline
                }
                Delete_Selected_Object
                Create_Canvas_Object $type $coords $style $colour $linewidth

                .can.c1 addtag $selectedAttribs withtag selected
            }
        }
    }
}

#============================================================================#

# Sets up the bindings for <ButtonRelease-1> in the canvas for the
# "Edit" functions.
# args :
#       1. A string identifying which operation has been selected, from the
#          users action.  Used to set up the correct binding.

proc Edit_Object_BR1 {operation} {

# The following switch command sets the appropriate command for <BR-1>.
    switch $operation {
        copySelected {
            bind .can.c1 <ButtonRelease-1> {
# Sets the end coordinates on the canvas, and calculates the difference.
                set end_x [.can.c1 canvasx %x]
                set end_y [.can.c1 canvasy %y]

                set dx [Calculate_Difference $start_x $end_x]
                set dy [Calculate_Difference $start_y $end_y]

# If the final-position is more than 5 pixels away, a copy is made.
# If not then the original object is reselected.
                if {$dx >= 5 || $dx <= -5 || $dy >= 5 || $dy <= -5} {
# Get the coordinate-list of the selected object.
                    set coords [.can.c1 coords selected]

# From the attribute-tag of the selected object, the attributes of the copy
# can be set to produce an exact copy.
# Identify the attribute-tag, and set appropriate variables to hold values.
                    set selectedAttribs [Find_Attributes $tagList]

                    set colour [lindex $selectedAttribs 1]
                    set linewidth [lindex $selectedAttribs 2]
                    if {$linewidth == 0} {
                        set style fill
                    } else {
                        set style outline
                    }

# Create a copy of the selected object over the original.
                    Create_Canvas_Object $type $coords $style $colour $linewidth
# And move the copy the calculated amount.
                    .can.c1 move selected $dx $dy
                    .can.c1 move visualaid $dx $dy
# Get a list of attributes for the copy and add to the tag-list.
                    set attributeList [Attribute_Tags selected]
                    .can.c1 addtag $attributeList withtag selected

# Remove all canvas binds for the moment
                    Nullify_All_Bindings
# If the "_Force_Annotation" option is selected, allow the user to annotate
# the new object.
                    if {$_Force_Annotation} {
                        Annotate_Selected
                        Configure_Test_URLbutton
                    }

                } else {
                    Remove_VisualAids
                    Auto_Select selected
                }
# Reset to the default canvas bindings after one cycle, if Single_Edit_Mode
# checkbutton is on.
                if {$_Single_Edit_Mode} {
                    Reset_To_Default_Binds
                } else {
                    Reset_To_Function_Binds
                }
            }
        }
        bothFlips {
            bind .can.c1 <ButtonRelease-1> {
                if {$_Single_Edit_Mode} {
                    Reset_To_Default_Binds
                }
            }
        }
    }
}

