#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to set bindings for all the "Create"       |#
#| object functions.  These include creating Rectangles and Circle type     |#
#| objects.                                                                 |#
#|                                                                          |#
#| Procedures :                                                             |#
#| 1. proc Edit_Polygon_BP1 - creates bindings for all <ButtonPress-1>      |#
#|                            sequences in the canvas, when editting        |#
#|                            polygons.                                     |#
#| 2. proc Edit_Polygon_B1M - creates bindings for all <B1-Motion> sequences|#
#|                            in the canvas, when editting polygons.        |#
#| 3. proc Edit_Polygon_BR1 - creates bindings for all <ButtonRelease-1>    |#
#|                            sequences in the canvas when editting         |#
#|                            polygons.                                     |#
#|__________________________________________________________________________|#
#                                                                            #

#=============================================================================#
# Invoked from "proc Edit_Objects" when the radiobutton selected is one of
# the 'polygon-specific' edit options.
# args :
#       1. String containing contents of global variable linked to Edit options.

proc Edit_Polygon_BP1 {operation} {

# The first part of the binding is common to all polygon-edit functions.
    .can.c1 bind visualaid <ButtonPress-1> {

        set editPolMode 1

# Identify the coordinates of the polygon, and the number of points.
        set coords [.can.c1 coords selected]
        set lengthCoords [expr ([llength $coords]-2)]

# Add the tag "chosen" to the point closest to the current x,y on canvas.
        .can.c1 addtag chosen closest [.can.c1 canvasx %x] \
            [.can.c1 canvasy %y] 1 "withtag visualaid"

        if {[lsearch [.can.c1 gettags chosen] current] == -1} {
            .can.c1 dtag chosen
        }

# Vars to hold point-coordinates on either side of chosen one.
        set prePt ""; set postPt ""

# From the coordinates of the chosen point, identify its position in the
# coordinate list of the polygon.
        set pointCo [.can.c1 coords chosen]

        set current_x [.can.c1 canvasx %x]
        set current_y [.can.c1 canvasy %y]

        set index 0
        while {$index < [llength $coords]} {
            set string "[lindex $coords $index] \
                [lindex $coords [expr $index+1]]"

            if {[string match [lrange $pointCo 0 1] $string]} {
                set index1 $index
                set index2 [expr $index+1]
            }
            incr index 2
        }
    }

# The switch command appends 'operation-specific' commands onto the
# previously set binding.  Which binding is set is dependant on the
# string held in "operation".
    switch $operation {
        MOVE_POINT {
            .can.c1 bind visualaid <ButtonPress-1> {+
                if {$index1==0} {
                    set prePt [list [lindex $coords 2] [lindex $coords 3]]
                    set postPt [list [lindex $coords [expr ($lengthCoords-2)]] \
                        [lindex $coords [expr ($lengthCoords-1)]]]
                } else {
                    set prePt [list [lindex $coords [expr ($index1-2)]] \
                        [lindex $coords [expr ($index1-1)]]]
                    set postPt [list [lindex $coords [expr ($index1+2)]] \
                        [lindex $coords [expr ($index1+3)]]]
                }
            }
        }
        ADD_POINT {
            .can.c1 bind visualaid <ButtonPress-1> {+
                set prePt [list [lindex $coords [expr ($index2+1)]] \
                    [lindex $coords [expr ($index2+2)]]]
                set postPt [list [lindex $coords $index1] \
                    [lindex $coords $index2]]
            }
        }
        DELETE_POINT {
            .can.c1 bind visualaid <ButtonPress-1> {+
# Amend the coordinate-list to not include the coordinates of the deleted point.
                if {$index1==0} {
                    set newx [lindex $coords [expr ($lengthCoords-4)]]
                    set newy [lindex $coords [expr ($lengthCoords-3)]]
                    set coords "$newx $newy \
                        [lrange $coords 2 [expr ($lengthCoords-3)]]"
                } else {
                    set ptsPre [lrange $coords 0 [expr ($index1-1)]]
                    set ptsPost [lrange $coords [expr ($index2+1)] \
                        [expr $lengthCoords+1]]

                    set coords "$ptsPre $ptsPost"
                }
            }
        }
    }
}

#=============================================================================#
# Set the binding for <B1-Motion> in the canvas.
# Craetes a visual-aid for the user - a line connecting points on either side
# of the chosen point going through the chosen one.
proc Edit_Polygon_B1M {} {

    bind .can.c1 <B1-Motion> {
        if {[info exists editPolMode]} {
# Delete the previous line
            .can.c1 delete visaid

# Calculate the difference in position after a cursor movement, and
# move the point the required amount
            set dx [Calculate_Difference $current_x %x]
            set dy [Calculate_Difference $current_y %y]

# Reset the current cursor position to the new position, so above
# calculation holds true for each movement
            set current_x [.can.c1 canvasx %x]
            set current_y [.can.c1 canvasy %y]

# Move command, and draw new connecting line
            .can.c1 move chosen $dx $dy
            eval ".can.c1 create line $prePt $current_x $current_y $postPt \
	        -fill $_Colour -tag visaid"
        }
    }
}

#=============================================================================#
# Invoked from "proc Edit_Objects" when the radiobutton selected is one of
# the 'polygon-specific' edit options.
# args :
#       1. String containing contents of global variable linked to Edit options.

proc Edit_Polygon_BR1 {operation} {

    bind .can.c1 <ButtonRelease-1> {
# If the Single_Edit_Mode checkbutton is on remove the edit polygon visual-aids.
            .can.c1 delete visaid

# Remove the tag "chosen" from the point with it.
        .can.c1 dtag chosen
    }

# Switch command invokes the binding for <ButtonRelease-1> specific to the
# option selected through radio-buttons.
    switch $operation {
        MOVE_POINT {
            bind .can.c1 <ButtonRelease-1> {+
                if {[info exists editPolMode]} {
# Amends the coordinate list changing the coordinate value of the moved point.
                    if {$index1==0} {
                        set coords [lreplace $coords 0 1 $current_x $current_y]
                        set coords [lreplace $coords $lengthCoords \
                            [expr ($lengthCoords+1)] $current_x $current_y]
                    } else {
                        set coords [lreplace $coords $index1 $index2 \
                            $current_x $current_y]
                    }
                }
            }
        }
        ADD_POINT {
            bind .can.c1 <ButtonRelease-1> {+
                if {[info exists editPolMode]} {
# Inserts the coordinates of the point added in the relevant list position.
                    if {$index1==0} {
                        set coords [linsert $coords 2 $current_x $current_y]
                    } else {
                        set coords [linsert $coords [expr ($index2+1)] \
                            $current_x $current_y]
                    }
                }
            }
        }
    }

# The following is set for both operations.
    bind .can.c1 <ButtonRelease-1> {+
        if {[info exists editPolMode]} {
# From the attribute-tag of the selected object, the attributes of the altered
# polygon can be set to produce a similar one.
            set attributes [Find_Attributes [.can.c1 gettags selected]]

            set objectColour [lindex $attributes 1]
            set objectWidth [lindex $attributes 2]

            Remove_VisualAids
            Delete_Selected_Object

            if {$objectWidth == 0} {
                set style fill
            } else {
                set style outline
                set coords [lrange $coords 0 [expr [llength $coords]-3]]
            }

# Create new object and tag appropriately.
            Create_Canvas_Object polygon $coords $style $objectColour \
                $objectWidth
            .can.c1 addtag $attributes withtag selected


            if {$_Single_Edit_Mode} {
                Reset_To_Default_Binds
            } else {
                set currentChoice $_Choice
                Reset_To_Default_Binds
                set _Choice $currentChoice
                Set_Correct_Binds $_Choice
            }
            unset editPolMode
        }
    }
}

