
#==========================================================================#
# This procedure is called whenever a file browser is required.
# The filename from the filebrowser is returned to here, then relevant
# procedures are called.
# args :
#       1. The command identifier.  Depending which menu-button is invoked,
#          will contain a different string.

proc File_Commands {command} {

    global _MapFile

# Get the filename returned from the file-browser
    set return [Create_Browser]

    set filter [lindex $return 0]
    set filename [lindex $return 1]

# Only if a filename has been returned, continue with the selected function.
    if {$filename != ""} {
# Get a string containing the full filename.
        set fullPath "$filter/$filename"

# Depending on the selected operation, call appropriate functions.
        switch $command {
            save {
                set _MapFile $fullPath

# If the Save_Map_As option has been chosen, and the file alraedy exists,
# make sure the user wants to overwrite that file.
                if {[file exists $_MapFile]} {
                    if {[yesNo "Overwrite existing file : $_MapFile ?"] \
                        == "NO"} {
                        return
                    }
                }
                Save_Map
            }
            image
                -
            map {
# For either loading images or loading/saving maps, the specified file must
# both exist and be readable by the current user.
                if {[Check_File $fullPath]} {
                    switch $command {
                        image {
                            Query_Image_Format $fullPath
                        }
                        map {
                            set _MapFile $fullPath
                            Read_Mapfile
                        }
                    }
                }
            }
        }
    }
}
#====================================================================#
# Check the current users privelages concerning the specified file.
# args    : 1. Full filename to check.
# returns : 0 - if file does not exist or the user cannot read it.
#           1 - if the current users privelages allow user to read it.

proc Check_File {filename} {

    if {![file exists $filename]} {
        postMessage "Cannot find file : $filename"
        return 0
    }

    if {![file readable $filename]} {
        postMessage "Cannot read file : $filename"
        return 0
    }
    return 1
}

#====================================================================#
# Create a new toplevel for the browser.

proc Create_Browser {} {

   global _Filter _Filename

    if {[winfo exists .fbrowser]} {
        destroy .fbrowser
    }

    toplevel .fbrowser
    wm title .fbrowser "Browser"

# Call procedure to label and create an entry-widget.
    Make_Entry .fbrowser.filter Filter

# Frame for the listboxes for directories and files.
    frame .fbrowser.list -relief raised -borderwidth 3
    pack append .fbrowser .fbrowser.list {top}
    frame .fbrowser.list.dir -relief sunken -borderwidth 3
    frame .fbrowser.list.files -relief sunken -borderwidth 3 
    pack append .fbrowser.list .fbrowser.list.dir {left} \
                               .fbrowser.list.files {left}

# Call procedure to make listboxes.
    Make_ListBox .fbrowser.list.dir Directories 
    Make_ListBox .fbrowser.list.files Files

# Call procedure to label and create an entry-widget.
    Make_Entry .fbrowser.filename Filename

# Call procedure to create the buttons.
    Make_Buttons

    Set_Browser_Binds

    Update_Display

    tkwait window .fbrowser

    return [list $_Filter $_Filename]
}
#====================================================================#
# Creates a frame containing a label and entry-widget.
# args : 1. Pathname to append on.
#        2. Textual string for label.  Also "_$labText" is the global
#           textvariable for the associated entry-box.

proc Make_Entry {frame labText} {

    frame $frame
    pack append .fbrowser $frame {top}

    label $frame.label -text $labText
    pack append $frame $frame.label {frame nw}

    entry $frame.entry -textvariable _$labText \
        -relief sunken -width 55
    pack append $frame $frame.entry {top}
}
#====================================================================#
# Creates a listbox and associated widgets - scrolbars (x and y) and a
# label.
# args : 1. Full pathname of the frame to contain associated widgets.
#        2. Textual string for label.

proc Make_ListBox {frame labText} {

    scrollbar $frame.scrolly -command "$frame.listbox yview"
    scrollbar $frame.scrollx -orient horiz -command "$frame.listbox xview"

    label $frame.label -text $labText
    pack append $frame $frame.label {frame nw}

    listbox $frame.listbox -relief sunken -geometry 25x10 \
        -yscroll "$frame.scrolly set" -xscroll "$frame.scrollx set" \
        -exportselection false

# Allow only one listbox element to be selected at a time.
    tk_listboxSingleSelect $frame.listbox

    pack append $frame $frame.scrollx {bottom fillx} \
                $frame.scrolly {right filly} \
                $frame.listbox {top expand fill}

}
    
#====================================================================#
# Create the buttons :
# 1. OK     - Select the pathname currently stored in "_Files".
# 2. Filter - Filter the directory/string currently in "_Directories".
# 3. Cancel - Kill the browser without selecting a file.

proc Make_Buttons {} {

    frame .fbrowser.buttons -relief raised -borderwidth 3
    pack append .fbrowser .fbrowser.buttons {top fillx}

    button .fbrowser.buttons.ok -text "OK" \
        -command {
            set _Filter [exec pwd]
            after 1000
            destroy .fbrowser
        }

    button .fbrowser.buttons.filter -text "Filter" \
        -command {
            Configure_Cursor .fbrowser working
            set curSelect [.fbrowser.list.dir.listbox curselection]
            if {$curSelect == ""} {
                set selected .
            } else {
                set selected [.fbrowser.list.dir.listbox get $curSelect]
            }
            Which_Filter $selected
        }
    button .fbrowser.buttons.cancel -text "Cancel" \
        -command {
            set _Filter ""
            set _Filename ""
            destroy .fbrowser
        }

    pack append .fbrowser.buttons \
                .fbrowser.buttons.ok {left pady 4 expand fillx} \
                .fbrowser.buttons.filter {left pady 4 expand fillx} \
                .fbrowser.buttons.cancel {left pady 4 expand fillx}
}

#====================================================================#
# Create bindings for the browser-box.

proc Set_Browser_Binds {} {

# 1. Double-clicking button-1 in the "directories" listbox is bound to
#    selecting a directory and pressing the "Filter" button.
    bind .fbrowser.list.dir.listbox <Double-1> {
        .fbrowser.buttons.filter invoke
    }

# 2. Pressing <Return> while the cursor is in the "Filter" entry box is bound
#    to pressing the "Filter" button with the contents of the entry-box,
#    if a "directory" listbox element is highlighted then this is appended on.
#    If not it is the same as "./".
#    This is convenient for "wild-carding".
    bind .fbrowser.filter.entry <Return> {
        .fbrowser.buttons.filter invoke
    }

# 3. Pressing button-1 on a "Files" listbox element, is bound to insert the
#    selected filename into the "Filename" entrybox.
    bind .fbrowser.list.files.listbox <Button-1> {
        set selected [Get_Listbox_Selection %W %y]
        Amend_File $selected
    }

# 4. Double-clicking button-1 in the "files" listbox is bound to pressing
#    the "OK" button.  The first button-press will insert the selected filename
#    into the "Filename" entry box.
    bind .fbrowser.list.files.listbox <Double-1> {
        .fbrowser.buttons.ok invoke
    }

# 5. Pressing <Return> while the cursor is in the "Filename" entrybox is
#    bound to pressing the "OK" button.
#    This is convenient when browser used to "Save_Map_As".
    bind .fbrowser.filename.entry <Return> {
        .fbrowser.buttons.ok invoke
    }

}

#====================================================================#
# Update the contents of the listboxes and entryboxes in the browser window.

proc Update_Display {{filePattern ""}} {

# Get current working directory.
    set cwd [exec pwd]

# Get the contents of the filter entry-box
    set filter [.fbrowser.filter.entry get]

    set filePattern [string range $filePattern 1 end]

# Empty the contents of the two listboxes and the filter entry-box.
    .fbrowser.filter.entry delete 0 end
    .fbrowser.filename.entry delete 0 end
    .fbrowser.list.dir.listbox delete 0 end
    .fbrowser.list.files.listbox delete 0 end

# Loop through a listing of all files in cwd.  If the file is a directory,
# insert it into the "Directory" listbox.  If it is a regular file, it is
# compared to the wild-card contained in "filePattern" if it is not a null
# string, and inserted into the "File" listbox.
    foreach file [exec ls -a $cwd] {
        if {[file isdirectory $file]} {
            .fbrowser.list.dir.listbox insert end $file
        } else {
            if {$filePattern != ""} {
                if {[string match [file extension $file] $filePattern]} {
                    .fbrowser.list.files.listbox insert end $file
                }
            } else {
                .fbrowser.list.files.listbox insert end $file
            }
        }
    }

# Insert the directory pathname into the "Filter" entrybox.
    .fbrowser.filter.entry insert 0 $cwd/*$filePattern
}

#====================================================================#
# When there is a need to change directory, this procedure identifies if the
# user has specified a "wild-card" string search for a file.
# Depending on this the procedure to update the display of the browser is
# called with relevant parameter.
proc Which_Filter {newDir} {

    set filter [split [.fbrowser.filter.entry get] /]

    set wildCard [lindex $filter [expr [llength $filter]-1]]

    if {$wildCard == ""} {
        cd $newDir
        Update_Display
    } else {
        cd $newDir
        Update_Display $wildCard
    }

    Configure_Cursor .fbrowser done
}
#====================================================================#
# Gets the listbox element closest to the current y coordinate of the cursor
# in the listbox.
# args    : 1. Pathname of the listbox cursor is in.
#           2. Y-coordinate of cursor when button was pressed.
# returns : String held in listbox under the cursor position.

proc Get_Listbox_Selection {listbox y} {

    $listbox select from [$listbox nearest $y]
    $listbox select to [$listbox nearest $y]

    return [$listbox get [$listbox nearest $y]]
}

#====================================================================#
# Deletes the contents and inserts the new selected filename into the
# filename entrybox.
# args : 1. String to be inserted in entrybox.
proc Amend_File {filename} {

    .fbrowser.filename.entry delete 0 end

    .fbrowser.filename.entry insert 0 $filename

}

